<?php
/**
 * Daisycon tracking pixel module
 *
 * @category  Daisycon
 * @package   Daisycon_Affiliatemarketing
 * @copyright Copyright (c) 2018 Daisycon B.V.
 * @author    Daisycon B.V. <support@daisycon.com>
 */
class Daisycon_Affiliatemarketing_Block_Tracking extends Daisycon_Affiliatemarketing_Block_Abstract
{
    /**
     * Generate the tracking url
     *
     * @return string
     */
    public function getCartSuccessPixelHtml()
    {
        if (false === $this->_getConfig()->isTrackingActive()) {
            return '';
        }

        $order = $this->_getOrder();
        if (null === $order) {
            return '';
        }

        $lccEnabled = $this->_getConfig()->isLccActive();
        $fromNetworkValue = Mage::getSingleton('core/cookie')->get($this->_getConfig()->getLccNetworkParameterName());

        if ($lccEnabled && $fromNetworkValue && $fromNetworkValue != "daisycon") {
            // If LCC is enabled, but the from network is SET and NOT `daisycon`.
            return '';
        }

        $helper = $this->helper('affiliatemarketing');

        $config = $this->_getConfig();

        $count = 0;

        $list = $this->_buildPartList($count);

        $billingAddress = $order->getBillingAddress();

        $version = (string)Mage::getConfig()->getNode('modules/Daisycon_Affiliatemarketing/version');

        $params = array(
            'ci' => $config->getGeneralProgramId(),
            'ti' => rawurlencode($order->getRealOrderId()),
            'cur' => $order->getOrderCurrencyCode(),
            'np' => $count,
            'c' => $billingAddress['country_id'],
            'z' => rawurlencode($billingAddress['postcode']),
            'gs' => $helper->getGender($order->getCustomerGender()),
            'gb' => $order->getCustomerDob(),
            'src' => 'magento1-' . $version,
        );

        if ($lccEnabled && !$fromNetworkValue) {
            // If LCC is enabled, but the from network is NOT SET (catch all).
            $params['ti'] = 'C' . $params['ti'];
        }

        $couponCodes = $this->_getItemCouponCodes();
        if ('' !== $order->getCouponCode()) {
            array_unshift($couponCodes, $order->getCouponCode());
        }

        if (false === empty($couponCodes)) {
            $params['pr'] = reset($couponCodes);
        }

        $url = '//' . $config->getTrackingSegment() . '/t/?';
        $url .= http_build_query($params);
        $url = $url . $list;

        $alt = 'Adverteren via Daisycon';
        $pixel = '<img src="' . $url . '" style="border: 0px; height: 1px; width: 1px;" '
            . 'alt="' . htmlentities($alt) . '" />';
        $pixel .= "<script type='text/javascript'>window.onload=function(){setTimeout(function(){"
            . "im=document.getElementsByTagName('img');for(i=0;i<im.length;i++){o=im[i];"
            . "if(/[s|c]i=/i.exec(o.src)&&(!o.offsetHeight||o.offsetHeight<1)){i=document.createElement('img');"
            . "i.height='1';i.width='1';i.id='news';i.className='net';"
            . "i.src='//'+Math.round(+new Date()/83000)+'.'+i.id+'tat.'+i.className+'/ab/'+"
            . "o.src.substring(o.src.indexOf('?'),o.src.length);document.body.appendChild(i);}}},100);}</script>";

        return $pixel;
    }

    /**
     * Build part list
     *
     * @param int $count
     * @return string
     * @throws Mage_Core_Exception
     * @throws Varien_Exception
     */
    protected function _buildPartList(&$count = 0)
    {
        $order = $this->_getOrder();
        if (null === $order) {
            return '';
        }

        $vatEnabled = (bool)$this->_getConfig()->getCompensationVat();
        $list = '';

        /**
         * @var Mage_Sales_Model_Order_Item $item
         */
        foreach ($order->getAllVisibleItems() as $item) {
            if (abs($item->getPriceInclTax()) < 0.001) {
                continue;
            }

            /**
             * @var Mage_Catalog_Model_Product $product
             */
            $product = Mage::getModel('catalog/product')->load($item->getProductId());

            $priceEach = (
                $item->getBaseRowTotal()
                + $item->getBaseTaxAmount()
                + $item->getBaseHiddenTaxAmount()
                + $item->getBaseWeeeTaxAppliedRowAmount()
                - $item->getBaseDiscountAmount()
            );

            if (false === $vatEnabled) {
                $priceEach /= 1 + $item->getTaxPercent() / 100;
            }

            $qty = $item->getQty() <> 0 ? $item->getQty() : $item->getQtyOrdered();
			if (1 > $qty) {
				$qty = 1;
			}
            $priceEach /= $qty;

            $data = array(
                'pn'  => rawurlencode($item->getName()),
                'a'   => number_format($priceEach * $qty, 2, '.', ''),
                'iv'  => rawurlencode($item->getName()),
                'r'   => number_format($priceEach * $qty, 2, '.', ''),
                'qty' => number_format($qty, 0, '.', ''),
                'sku' => $item->getSku(),
                'cc'  => rawurlencode($this->_getCompensationCode($product)),
            );

            $data = array_merge($data, $this->_getExtra($product));

            $list .= '&p[]=';
            foreach ($data as $key => $param) {
                $list .= '{' . $key . ':' . $param . '}';
            }

            ++$count;
            unset($product);
        }

        return $list;
    }

    /**
     * Get all used coupon codes
     *
     * @return array
     */
    protected function _getItemCouponCodes()
    {
        $order = $this->_getOrder();
        if (null === $order) {
            return array();
        }

        $couponCodes = array();
        foreach ($order->getAllVisibleItems() as $item) {
            if (abs($item->getPrice()) < 0.001) {
                continue;
            }

            if ('' !== $item->getAppliedRuleIds()) {
                foreach (explode(',', $item->getAppliedRuleIds()) as $ruleId) {
                    // Load the rule object
                    $rule = Mage::getModel('salesrule/rule')->load($ruleId);
                    // Throw out some information like the rule name what product it was applied to
                    $couponCodes[] = $rule->getCouponCode();
                }
            }
        }

        return $couponCodes;
    }

    /**
     * Get compensation code
     *
     * @param Mage_Catalog_Model_Product $product
     * @throws Varien_Exception
     * @return string|null
     */
    protected function _getCompensationCode(Mage_Catalog_Model_Product $product)
    {
        $config = $this->_getConfig();
        $helper = $this->helper('affiliatemarketing');
        switch ($config->getCompensationField())
        {
            case Daisycon_Affiliatemarketing_Model_System_Config_Source_Compensation_Field::PRODUCT_ATTRIBUTE:
                $attribute = $config->getCompensationFieldProductAttribute();
                if (false === empty($field)) {
                    return $helper->getProductValue($product, $attribute);
                }
                return null;

            case Daisycon_Affiliatemarketing_Model_System_Config_Source_Compensation_Field::CATEGORY_ID:
                return $helper->getCategoryId($product);

            default:
                return null;
        }
    }


    /**
     * Get extra field values
     *
     * @param $product
     * @return array
     * @throws Varien_Exception
     * @throws Mage_Core_Exception
     */
    protected function _getExtra($product)
    {
        $config = $this->_getConfig();
        $helper = $this->helper('affiliatemarketing');

        $extra = $config->getTrackingExtra();
        $extraProductAttributes = $config->getTrackingExtraProductAttribute();

        $data = array();

        foreach ($extra as $extraIndex => $extraField) {
            if (true === empty($extraField)) {
                continue;
            }

            switch ($extraField) {
                case Daisycon_Affiliatemarketing_Model_System_Config_Source_Tracking_Extra::PRODUCT_ATTRIBUTE:
                    if (false === empty($extraProductAttributes[$extraIndex])) {
                        $extraValue = $helper->getProductValue($product, $extraProductAttributes[$extraIndex]);
                        break;
                    }
                    continue;

                case Daisycon_Affiliatemarketing_Model_System_Config_Source_Tracking_Extra::CATEGORY_ID:
                    $extraValue = $helper->getCategoryId($product);
                    break;

                case Daisycon_Affiliatemarketing_Model_System_Config_Source_Tracking_Extra::PAYMENT_METHOD:
                    $extraValue = $this->_getOrder()->getPayment()->getMethodInstance()->getTitle();
                    break;

                case Daisycon_Affiliatemarketing_Model_System_Config_Source_Tracking_Extra::SHIPPING_METHOD:
                    $extraValue = $this->_getOrder()->getShippingDescription();
                    break;
            }

            $data['e' . $extraIndex] = rawurlencode($extraValue);
        }

        return $data;
    }
}
